module Jqgrid_Helper

  include ActionView::Helpers::JavaScriptHelper

  def to_json_for_jqgrid_with_firms_and_attorneys_for_case(current_page, per_page, total)
    json = %Q({"page":"#{current_page}","total":#{total/per_page.to_i+1},"records":"#{total}")
    if total > 0
      json << %Q(,"rows":[)
      each do |acase|
        json << %Q({"id":"#{acase.id}","cell":[)

        # note: must repeat "id" column within cell array:
        value = acase.id
        value = escape_javascript(value) if value and value.is_a? String
        json << %Q("#{value}",)
        
        value = acase.name
        value = escape_javascript(value) if value and value.is_a? String
        json << %Q("#{value}",)
        
        case_firm = acase.firm
        value = case_firm.blank? ? '' : case_firm.id
        value = escape_javascript(value) if value and value.is_a? String
        json << %Q("#{value}",)
        
        # handle none, single, or multiple attorneys assigned to a case so jqgrid can create a proper multi-select html element:
        json << %Q([)
        attorneys = acase.users
        if attorneys.length <= 0
          json << %Q("",)
        elsif
          attorneys.each do |attorney|
            json << %Q("#{attorney.id}",)
          end
        end
        json.chop! << %Q(]) # chop! off last char which is a comma

        json << "]},"
      end
      json.chop! << "]}" # chop! off last char which is a comma
    else
      json << "}"
    end
  end

  def to_json_for_jqgrid(attributes, current_page, per_page, total)
    # ActiveRecord::Base.logger.info ">>>>>>>>>>>>>>>>>> to_json_for_jqgrid ......................"
    json = %Q({"page":"#{current_page}","total":#{total/per_page.to_i+1},"records":"#{total}")
    if total > 0
      json << %Q(,"rows":[)
      each do |elem|
        elem.id ||= index(elem)
        json << %Q({"id":"#{elem.id}","cell":[)
        couples = elem.attributes.symbolize_keys
        attributes.each do |atr|
          value = get_atr_value(elem, atr, couples)
          value = escape_javascript(value) if value and value.is_a? String
          json << %Q("#{value}",)
        end
        json.chop! << "]},"
      end
      json.chop! << "]}"
    else
      json << "}"
    end
  end
  
  private
  
  def get_atr_value(elem, atr, couples)
    if atr.to_s.include?('.')
      value = get_nested_atr_value(elem, atr.to_s.split('.').reverse) 
    else
      value = couples[atr]
      value = elem.send(atr.to_sym) if value.blank? && elem.respond_to?(atr) # required for virtual attributes
    end
    value
  end
  
  def get_nested_atr_value(elem, hierarchy)
    return nil if hierarchy.size == 0
    atr = hierarchy.pop
    raise ArgumentError, "#{atr} doesn't exist on #{elem.inspect}" unless elem.respond_to?(atr)
    nested_elem = elem.send(atr)
    return "" if nested_elem.nil?
    value = get_nested_atr_value(nested_elem, hierarchy)
    value.nil? ? nested_elem : value
  end
end